//
// main.cxx -- test the Fl_Native_File_Chooser widget
//
// Copyright 2004 by Greg Ercolano.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Value_Input.H>

//#include "Fl_Native_File_Chooser_FLTK.H"
//#include "Fl_Native_File_Chooser_WIN32.H"
#include "Fl_Native_File_Chooser_MAC.H"

#ifdef WIN32
// WINDOWS //
#include <direct.h>
#define getcwd _getcwd
#define MAXPATHLEN MAX_PATH
#else
// UNIX //
#include <sys/param.h>
#include <unistd.h>
#endif

// GLOBALS
Fl_Native_File_Chooser *G_choo = NULL;
Fl_Input               *G_inpdirname = NULL;
Fl_Input               *G_inptitle = NULL;
Fl_Input               *G_inpfilen = NULL;
Fl_Multiline_Input     *G_inpfilter = NULL;
Fl_Value_Input         *G_nfilter = NULL;

void Butt_CB(Fl_Widget*w, void*data) {

    if ( G_inpdirname->value()[0] ) G_choo->directory(G_inpdirname->value());
    else G_choo->directory(NULL);

    if ( G_inpfilen->value()[0] ) G_choo->preset_file(G_inpfilen->value());
    else G_choo->preset_file(NULL);

    if ( G_inptitle->value()[0] ) G_choo->title(G_inptitle->value());
    else G_choo->title("");

    if ( G_inpfilter->value()[0] ) G_choo->filter(G_inpfilter->value());
    else G_choo->filter("");

    if ( G_nfilter->value() ) G_choo->set_filter((int)G_nfilter->value());
    else G_choo->set_filter(0);

    switch ( G_choo->show() ) {
        case -1:	// ERROR
	    fprintf(stderr, "*** ERROR show():%s\n", G_choo->errmsg());
	    break;
        case 1:		// CANCEL
	    fprintf(stderr, "*** CANCEL\n");
	    break;

	default:
	    break;
    }
	fprintf(stderr,"\n %s\n",G_choo->errmsg() );
    fprintf(stderr,"Filter: %i of %i\n",G_choo->set_filter(),G_choo->filters());

    int count = G_choo->count();

	fprintf(stderr,"Total chosen %i\n",count);
	
    if (count > 1) {
	 for ( int t=0; t<count; t++ ) {
	    fprintf(stderr, "MULTI FILENAMES RESULT %d) '%s'\n", t, G_choo->filename(t));
	 }
    }
    else if (count == 1) {
	 fprintf(stderr, "SINGLE FILENAME RESULT: '%s'\n", G_choo->filename());
    }
    fflush(stderr);  // MSYS wasn't displaying stderr until close
}

void choice_cb(Fl_Widget*, void *data) {
    Fl_Choice *o = (Fl_Choice*)data;
    switch (o->value()) {
        case 0: G_choo->type(Fl_Native_File_Chooser::BROWSE_FILE);            break;
        case 1: G_choo->type(Fl_Native_File_Chooser::BROWSE_DIRECTORY);       break;
        case 2: G_choo->type(Fl_Native_File_Chooser::BROWSE_MULTI_FILE);      break;
        case 3: G_choo->type(Fl_Native_File_Chooser::BROWSE_MULTI_DIRECTORY); break;
        case 4: G_choo->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);       break;
        case 5: G_choo->type(Fl_Native_File_Chooser::BROWSE_SAVE_DIRECTORY);  break;
    }
}

int main() {
     char start_dir[MAXPATHLEN + 1];
     getcwd(start_dir, MAXPATHLEN);

     Fl_Window *win = new Fl_Window(300, 230,"Fl_Native_File_Chooser");

     Fl_Choice *cho = new Fl_Choice(10,10,130,25);
     cho->add("Single File",      (const char*)NULL, choice_cb, (void*)cho);
     cho->add("Single Directory", (const char*)NULL, choice_cb, (void*)cho);
     cho->add("Multi File",       (const char*)NULL, choice_cb, (void*)cho);
     cho->add("Multi Directory",  (const char*)NULL, choice_cb, (void*)cho);
     cho->add("Save File",        (const char*)NULL, choice_cb, (void*)cho);
     cho->add("Save Directory",   (const char*)NULL, choice_cb, (void*)cho);
     cho->value(0);

     Fl_Button *but = new Fl_Button(150,10,80,25,"Browser");
     but->callback(Butt_CB);

     G_inptitle = new Fl_Input(80,50,200,25,"Title");
     G_inptitle->value("Title Of Window");

     G_inpdirname = new Fl_Input(80,80,200,25,"Preset Dir");
     G_inpdirname->value(start_dir);

     G_inpfilen = new Fl_Input(80,110,200,25,"Filename");
     G_inpfilen->value("name.txt");

     G_inpfilter = new Fl_Multiline_Input(80,145,200,50,"Filter");
     G_inpfilter->value("");

     G_nfilter = new Fl_Value_Input(80,200,200,25,"Filter index");
     G_nfilter->value(0);

     G_choo = new Fl_Native_File_Chooser();
     G_choo->type(Fl_Native_File_Chooser::BROWSE_FILE);

     win->show();
     return(Fl::run());
}
